
/* *************************************************************************
 *  File : Tx.c
 * -------------------------------------------------------------------------
 *  All rights are reserved by Bo-kyun Na, IntellActive Research Inc.
 * -------------------------------------------------------------------------
 * This example reads an image from /dev/video, transmits compressed video 
 * over the local network, and optionally displays the image in Xwindow.
 * ************************************************************************* */

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <semaphore.h>

#include "params.h"
#include "network.h"
#include "transceive.h"
#include "vdo.h"
#include "h261.h"

int expand;
int verbose = 0;

void error ();
void init_dither_tab ();
void YUV411toRGB16();
void YUV411toRGB24();
void YUV411toRGB32 ();
void YUV411toRGB24sol ();


/* *************************************************************************
 * void *Thread_TxVideo (void *arg)
 *
 *
 * ************************************************************************* */
int main (int argc, char **argv)
{
  int i;

  Parameters *param;
  TransceiveState *tstate;

  int vid;
  char	devname[64];
  unsigned char *capimg;
  XWA xwa;
  V4L2 v4l2;
  VIMAGE vimage;

  int first = 0;
  int CISsize;
  char inbuf[VDO_WIDTH * VDO_HEIGHT * 3/2]; 
  char outbuf[VDO_WIDTH * VDO_HEIGHT * 3/2];
  char tmpbuf[VDO_WIDTH * VDO_HEIGHT * 3/2];
  unsigned char *y, *u, *v;
  unsigned char *y1, *u1, *v1;
  int userno = 0;

  /* Put in the device node name */
  strcpy(devname, VDO_DEVICE);

  /* Initialization of network parameters */
  param = (Parameters *) init_Param (argc, argv);

  /* Initialization of transmission parameters */
  if (param->transmit)
    tstate = (TransceiveState *) initialize_transmit(param, VIDEO);

  /* Pointers for the conversion of RGBtoYUV */
  y = inbuf;
  u = &inbuf[VDO_WIDTH * VDO_HEIGHT];
  v = &inbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];

  y1 = tmpbuf;
  u1 = &tmpbuf[VDO_WIDTH * VDO_HEIGHT];
  v1 = &tmpbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];
  
  xwa.xdepth = 0;		/* init. of depth on xwindow */      
  v4l2.fmt.fmt.pix.depth = 16;

  /* Process command line arguments */
  for (i = 1; i < argc; i++) {
    if (!strcmp (argv[i], "-C")) {
      if (!strcmp(argv[++i], "H261")) {
      	param->codec = H261;
      	param->cis_type = "H261";
      }
      else if (!strcmp(argv[i], "H263")) {
      	param->codec = H263;
      	param->cis_type = "H263";
      }
      else if (!strcmp(argv[i], "JPEG")) {
      	param->codec = JPEG;
      	param->cis_type = "Jpeg";
      }
      else if (!strcmp(argv[i], "MPEG")) {
      	param->codec = MPEG;
      	param->cis_type = "Mpeg1";
      }
      else if (!strcmp(argv[i], "RAW")) {
	param->codec = RAW;
      	param->cis_type = "RAW";
      }
      else {
	printf ("Specifiy your codec mode\n");
	exit (0);
      }
    }
    if(!strcmp(argv[i], "-D")) {
      strcpy (devname, argv[++i]);
    }
    if(!strcmp(argv[i], "-r")) {
      //      rate = atoi(argv[++i]);
    }
    if(!strcmp(argv[i], "-s")) {
      //      width = atoi(argv[++i]);
      //      height = atoi(argv[++i]);
    }
    if(!strcmp(argv[i], "-v")) {
      verbose = 1;
    }
    if(!strcmp(argv[i], "-x")) {
      xwa.xdepth = atoi(argv[++i]);
      if (verbose)
	printf ("The depth of captured images is %d.\n", 
		v4l2.fmt.fmt.pix.depth);
    }
  }

  printf ("codec is %s\n", param->cis_type);

  /* Function: Init. of video capture to get the video from my camera */
  vid = BV_init_vidcap (devname, &v4l2);
  capimg = (unsigned char *) malloc (v4l2.fmt.fmt.pix.sizeimage);

  /* Init. and format of xwindow display */
  vimage.width = VDO_WIDTH;
  vimage.height = VDO_HEIGHT;
  BV_init_display (&xwa, argc, argv, &vimage);

  init_codec ();		/* Initialization of codec of H261 */
  init_encoder ();		/* Initilization of H261 encoder */
  init_decoder ();

  init_dither_tab ();

  /* Loop processing related to a video frame */
  for (;;) {
      BV_get_vdoframe (vid, capimg, &v4l2);

    /* Packetization and sending of video data */
    if (param->transmit) {
      if (param->codec == H261) {
	/* Conversion of RGB16 into YUV411 */
	RGB16toYUV(capimg, y, u, v, VDO_WIDTH, VDO_HEIGHT);

	/* compression */
	//CISsize = ((first %= 10) == 0) ? p64EncodeIntraFrame (inbuf, outbuf):
	//                               p64EncodeFrame (inbuf, outbuf);

	/* transmit compressed image sequence data */
	//BV_packetize_vdoframe(outbuf,CISsize,VDO_WIDTH,VDO_HEIGHT,tstate,param);

	/* Decode of recvd message */
	//decode_frame (outbuf, CISsize, tmpbuf, (first == 0) ? 1 : 0, userno);

	if ((xwa.xdepth == 24) || (xwa.xdepth == 32)) {
	  // YUV411toRGB32 (y1, u1, v1, vimage.data, VDO_WIDTH, VDO_HEIGHT);	
	  YUV411toRGB32 (y, u, v, vimage.data, VDO_WIDTH, VDO_HEIGHT);
	  //  YUV411toRGB24 (y, u, v, vimage.data,VDO_WIDTH, VDO_HEIGHT);
	  //  YUV411toRGB24 (y, u, v, vimage.data, VDO_WIDTH, VDO_HEIGHT);
	  //  YUV411toRGB24sol (y, u, v, vimage.data, VDO_WIDTH, VDO_HEIGHT);
	}
	else if (xwa.xdepth == 16) { /* Color Conversion YUV411 to RGB16 */
	  YUV411toRGB16(y1, u1, v1, vimage, 24);
	}

	first++;  
      }
      else if (param->codec == H263) {
	printf ("Not available.\n");
	exit (0);
      }
      else if (param->codec == JPEG) {
	printf ("Not available.\n");
	exit (0);
      }
      else if (param->codec == MPEG) {
	printf ("Not available.\n");
	exit (0);
      }
      else if (param->codec == RAW) {	/* transmit raw video data */

	BV_packetize_vdoframe (capimg, v4l2.fmt.fmt.pix.sizeimage, 
			       VDO_WIDTH, VDO_HEIGHT, tstate, param);

	memcpy (vimage.data, capimg, v4l2.fmt.fmt.pix.sizeimage);
      }
      else {
	printf ("Error on sending. Specifiy a correct codec type.\n");
	exit (0);
      }
      /* Display compressed local images */
      BV_display_vdoframe (&xwa, &vimage);
    }
  }
}
